/*
This is a part of the mzscript LiteStep module source code.

Copyright (C) 2000-2003 The mzscript Development Team (see mzscript.txt)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _MZSCRIPT_H
#define _MZSCRIPT_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "common.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// typedefs
//
typedef std::pair<tstring, tstring> Line;
typedef std::vector<Line> LineVector;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class ParserError (for exception handling)
//
class ParserError : public std::invalid_argument
{
    bool bWasDisplayed;

public:
    ParserError(const std::string& sMessage) :
      invalid_argument(sMessage), bWasDisplayed(false) {}

    void SetDisplayed() { bWasDisplayed = true; }
    bool WasDisplayed() const { return bWasDisplayed; }
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Function prototypes
//
void LoadSetup(bool bRefresh);
void ClearSetup();
bool ParseScriptFile(const TCHAR* ptzFileName, LineVector* plvRest);
void ParseVarFile(const TCHAR* ptzFileName);
void ExecCommands(LPCTSTR pszKey, const LineVector& lvContainer);

void ExecCommand(const tstring& sCommand);
bool EvalIf(tstring& sStatement);


LRESULT CALLBACK MessageHandler(HWND hWnd, UINT uMsg, WPARAM wParam,
                                LPARAM lParam);


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Exports
//
#ifdef  __cplusplus
extern "C"
{
#endif  // __cplusplus

    __declspec( dllexport ) int initModuleEx(HWND hLitestep, HINSTANCE hInstance, LPCSTR pszPath);
	__declspec( dllexport ) void quitModule(HINSTANCE hInstance);

#ifdef  __cplusplus
}
#endif  // __cplusplus

#endif // _MZSCRIPT_H