/*
This is a part of the mzscript LiteStep module source code.

Copyright (C) 2000-2003 The mzscript Development Team (see mzscript.txt)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _MZSCRIPT_SCRIPTS_H
#define _MZSCRIPT_SCRIPTS_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "common.h"
#include "variables.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class Script
//
class Script
{
public:
    enum LINETYPE { EXEC, LABEL, GOTO, GOTOIF, EXIT };
    
    Script(const tstring& sName) : m_sName(sName) {}
    
    // AddLine can throw ParseError
    void AddLine(LPCTSTR ptzType, const tstring& sRest);
    void Execute(Variables::Ptr pVarContext);

    const tstring& GetName() const { return m_sName; }

private:
    typedef std::pair<LINETYPE, tstring> ScriptLine;
    typedef std::vector<ScriptLine> ScriptSource;
    
    tstring m_sName;
    ScriptSource m_Source;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// typedefs
//
typedef boost::shared_ptr<Script> ScriptPtr;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Function prototypes
//
void ScriptAdd(ScriptPtr pScript);
void ScriptRemove(const tstring& sName);

//ScriptPtr ScriptGet(const tstring& sName);

void ScriptsClear();
size_t ScriptCount();

void ScriptBangHandler(HWND hCaller, const char* pszName, const char* pszArgs);

   
#endif // _MZSCRIPT_SCRIPTS_H