/*
This is a part of the mzscript LiteStep module source code.

Copyright (C) 2000-2003 The mzscript Development Team (see mzscript.txt)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "utility.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals
//
//
extern const TCHAR* g_tzAppName;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetLastErrorStr
//
tstring GetLastErrorStr()
{
    void* pMsgBuf = NULL;
    
    // TODO: both FormatMessage and LocalFree can fail... and they both
    // overwrite the GetLastError() flag... sigh
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        GetLastError(),
        0,
        (LPTSTR)&pMsgBuf,
        0,
        NULL
        );

    assert(pMsgBuf);
    tstring sReturn((LPTSTR)pMsgBuf);
    
    LocalFree(pMsgBuf);
    return sReturn;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReportLastError
//
void ReportLastError()
{
    ErrorMessage(GetLastErrorStr());
}

void ReportLastError(LPCTSTR ptzAdditional)
{
    ErrorMessagef("%s.\nDescription: %s",
        ptzAdditional, GetLastErrorStr().c_str());
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ErrorMessage
//
void ErrorMessage(const tstring& sMessage)
{
    MessageBox(NULL, sMessage.c_str(), g_tzAppName,
        MB_TOPMOST | MB_SETFOREGROUND | MB_ICONERROR);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ErrorMessagef
//
void ErrorMessagef(LPCTSTR ptzFormat, ...)
{
    TCHAR tzMessage[MAX_LINE_LENGTH];
    va_list argList;
    
    va_start(argList, ptzFormat);
    StringCchVPrintf(tzMessage, MAX_LINE_LENGTH, ptzFormat, argList);
    va_end(argList);
    
    ErrorMessage(tzMessage);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// AskUser
//
int AskUser(UINT uFlags, LPCTSTR ptzFormat, ...)
{
    TCHAR tzMessage[MAX_LINE_LENGTH];
    va_list argList;
    
    va_start(argList, ptzFormat);
    StringCchVPrintf(tzMessage, MAX_LINE_LENGTH, ptzFormat, argList);
    va_end(argList);

    return MessageBox(NULL, tzMessage, g_tzAppName,
        MB_TOPMOST | MB_SETFOREGROUND | uFlags);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// AddBangCommands/RemoveBangCommands
//
// add/remove bang command list
//
void AddBangCommands(const BangCommandAssoc* pBangs)
{
    assert(pBangs != NULL);
    
    // go through a NULL-terminated list of bang commands...
    while (pBangs->pszName != NULL)
    {
        // and register them individually
        AddBangCommand(pBangs->pszName, pBangs->pbCommand);
        // proceed in the list
        pBangs++; // FIXME: use separate pointer and const*const?
    }
}

void RemoveBangCommands(const BangCommandAssoc* pBangs)
{
    assert(pBangs != NULL);
    
    // go through a NULL-terminated list of bang commands...
    while (pBangs->pszName != NULL)
    {
        // and remove them individually
        RemoveBangCommand(pBangs->pszName);
        // proceed in the list
        pBangs++;
    }
}

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetToken
//
bool GetToken(tstring& sToken, tstring& sString, bool bUseBrackets)
{
    if (sString.empty())
    {
        return false;
    }
    
    char szToken[MAX_LINE_LENGTH];
    
    const TCHAR* pszOld = sString.c_str();
    const TCHAR* pszNew = pszOld;
    
    bool bReturn = GetToken(pszOld, szToken, &pszNew, bUseBrackets) ? true : false;
    
    if (!pszNew)
    {
        sString.clear();
    }
    else
    {
        sString.erase(0, pszNew - pszOld);
    }
    
    sToken.assign(szToken);
    
    return bReturn;
}
