/*
This is a part of the mzscript LiteStep module source code.

Copyright (C) 2000-2003 The mzscript Development Team (see mzscript.txt)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _MZSCRIPT_VARIABLES_H
#define _MZSCRIPT_VARIABLES_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "utility.h"
#include "common.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class Variable
//
class Variable
{
public:
    typedef tstring (*Func)();

    Variable(const tstring& sValue, const tstring& sFile) :
      m_sValue(sValue), m_pFunc(NULL), m_sFile(sFile), m_cSeparator(_T(':')) {}
      
    Variable(Func pFunc) :
      m_pFunc(pFunc), m_cSeparator(_T(':'))
        { assert(!IsBadCodePtr((FARPROC)m_pFunc)); }
      
    ~Variable() {}
     
    tstring Get(int nIndex) const;
    void Set(const tstring& sValue, int nIndex);
    bool Remove(int nIndex);

    void SetSeparator(TCHAR cSeparator) { m_cSeparator = cSeparator; }
    const tstring& GetFile() const
    {
        return m_sFile;
    }
      
private:
    int _GetElementPos(int nIndex, size_t& stStart, size_t& stEnd) const;

    TCHAR m_cSeparator;
    tstring m_sFile;
    mutable tstring m_sValue;
    Func m_pFunc;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class Variables
//
// Currently only GetValue is aware of m_pParent
//
class Variables
{
public:
    typedef boost::shared_ptr<Variables> Ptr;
    typedef boost::shared_ptr<Variable> VarPtr;
    
    explicit Variables(Ptr pParent) : m_pParent(pParent) {}
    ~Variables(){}
    
    bool Expand(tstring& sString) const;

    bool Get(tstring sVariable, tstring* psValue) const;
    VarPtr Set(tstring sVariable, const tstring& sValue,
             const tstring& sFile = tstring());
    bool Set(const tstring& sVariable, Variable::Func pFunc);

    bool Remove(tstring sVariable);
    
    void Clear() { m_VarMap.clear(); }
    size_t GetCount() const { return m_VarMap.size(); }
    
private:
    typedef std::map<tstring, VarPtr, tstringicmp> VarMap;
    
    bool _GetValueWorker(const tstring& sVariable, tstring* psValue,
        int nIndex) const;
    bool _ExpandVarsWorker(tstring& sString) const;
    int _InterpretVar(tstring& sVariable) const;

    Ptr m_pParent;
    VarMap m_VarMap;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// "exports"
//
extern Variables::Ptr g_Variables;


#endif // _MZSCRIPT_VARIABLES_H